use serde::{Deserialize, Serialize};
use std::fmt::{Display, Formatter};
use std::net::SocketAddr;
use domo_proto::packet::identifier::Identifier;

#[derive(Debug, Serialize, Deserialize, Clone)]
pub enum NodeType {
    #[serde(rename = "master")]
    Master {
        bind: SocketAddr,
        identifier: String
    },
    #[serde(rename = "relay")]
    Relay {
        bind: SocketAddr,
        master_address: String
    },
    #[serde(rename = "subnet")]
    Subnet {
        master_address: String
    },
}

impl Display for NodeType {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            NodeType::Master { .. } => write!(f, "master"),
            NodeType::Relay { .. } => write!(f, "relay"),
            NodeType::Subnet { .. } => write!(f, "subnet"),
        }
    }
}

impl Default for NodeType {
    fn default() -> Self {
        Self::Master {
            bind: SocketAddr::from(([0,0,0,0], 4480)),
            identifier: hex::encode(Identifier::random().to_string())
        }
    }
}

#[derive(Debug, Serialize, Deserialize, Default, Clone)]
pub struct NodeConfig {
    #[serde(rename = "type")]
    pub node_type: NodeType,
    pub device_id: Option<String>
}
